@@ -52,19 +52,19 @@ extern "C" {
 using namespace std;
 
 // Perform t-SNE
-void TSNE::run(double* X, int N, int D, double* Y, int no_dims, double perplexity, double theta) {
+void TSNE::run(double* X, int N, int D, double* Y, int no_dims, double perplexity, double theta, bool verbose, int max_iter, double* cost, bool distance_precomputed, double* itercost) {
     
     // Determine whether we are using an exact algorithm
     if(N - 1 < 3 * perplexity) { Rcpp::stop("Perplexity too large for the number of data points!\n"); }
-    Rprintf("Using no_dims = %d, perplexity = %f, and theta = %f\n", no_dims, perplexity, theta);
+    if (verbose) Rprintf("Using no_dims = %d, perplexity = %f, and theta = %f\n", no_dims, perplexity, theta);
     bool exact = (theta == .0) ? true : false;
     
     // Set learning parameters
     float total_time = .0;
     clock_t start, end;
-	int max_iter = 1000, stop_lying_iter = 250, mom_switch_iter = 250;
-	double momentum = .5, final_momentum = .8;
-	double eta = 200.0;
+	  int stop_lying_iter = 250, mom_switch_iter = 250;
+	  double momentum = .5, final_momentum = .8;
+	  double eta = 200.0;
     
     // Allocate some memory
     double* dY    = (double*) malloc(N * no_dims * sizeof(double));
@@ -75,14 +75,17 @@ void TSNE::run(double* X, int N, int D, double* Y, int no_dims, double perplexit
     for(int i = 0; i < N * no_dims; i++) gains[i] = 1.0;
     
     // Normalize input data (to prevent numerical problems)
-    Rprintf("Computing input similarities...\n");
+    if (verbose) Rprintf("Computing input similarities...\n");
     start = clock();
-    zeroMean(X, N, D);
-    double max_X = .0;
-    for(int i = 0; i < N * D; i++) {
-        if(X[i] > max_X) max_X = X[i];
+    if (!distance_precomputed) {
+      if (verbose) Rprintf("Normalizing input...\n");
+      zeroMean(X, N, D);
+      double max_X = .0;
+      for(int i = 0; i < N * D; i++) {
+          if(X[i] > max_X) max_X = X[i];
+      }
+      for(int i = 0; i < N * D; i++) X[i] /= max_X;
     }
-    for(int i = 0; i < N * D; i++) X[i] /= max_X;
     
     // Compute input similarities for exact t-SNE
     double* P; int* row_P; int* col_P; double* val_P;
@@ -91,10 +94,10 @@ void TSNE::run(double* X, int N, int D, double* Y, int no_dims, double perplexit
         // Compute similarities
         P = (double*) malloc(N * N * sizeof(double));
         if(P == NULL) { Rcpp::stop("Memory allocation failed!\n"); }
-        computeGaussianPerplexity(X, N, D, P, perplexity);
+        computeGaussianPerplexity(X, N, D, P, perplexity, distance_precomputed);
     
         // Symmetrize input similarities
-        Rprintf("Symmetrizing...\n");
+        if (verbose) Rprintf("Symmetrizing...\n");
         for(int n = 0; n < N; n++) {
             for(int m = n + 1; m < N; m++) {
                 P[n * N + m] += P[m * N + n];
@@ -110,7 +113,7 @@ void TSNE::run(double* X, int N, int D, double* Y, int no_dims, double perplexit
     else {
     
         // Compute asymmetric pairwise input similarities
-        computeGaussianPerplexity(X, N, D, &row_P, &col_P, &val_P, perplexity, (int) (3 * perplexity));
+        computeGaussianPerplexity(X, N, D, &row_P, &col_P, &val_P, perplexity, (int) (3 * perplexity), verbose, distance_precomputed);
         
         // Symmetrize input similarities
         symmetrizeMatrix(&row_P, &col_P, &val_P, N);
@@ -128,9 +131,12 @@ void TSNE::run(double* X, int N, int D, double* Y, int no_dims, double perplexit
 	for(int i = 0; i < N * no_dims; i++) Y[i] = randn() * .0001;
 	
 	// Perform main training loop
+  if (verbose) {
     if(exact) Rprintf("Done in %4.2f seconds!\nLearning embedding...\n", (float) (end - start) / CLOCKS_PER_SEC);
     else Rprintf("Done in %4.2f seconds (sparsity = %f)!\nLearning embedding...\n", (float) (end - start) / CLOCKS_PER_SEC, (double) row_P[N] / ((double) N * (double) N));
-    start = clock();
+  }
+  start = clock();
+  int costi = 0; //iterator for saving the total costs for the iterations
 	for(int iter = 0; iter < max_iter; iter++) {
         
         // Compute (approximate) gradient
@@ -161,17 +167,24 @@ void TSNE::run(double* X, int N, int D, double* Y, int no_dims, double perplexit
             double C = .0;
             if(exact) C = evaluateError(P, Y, N, no_dims);
             else      C = evaluateError(row_P, col_P, val_P, Y, N, no_dims, theta);  // doing approximate computation here!
-            if(iter == 0)
-                Rprintf("Iteration %d: error is %f\n", iter + 1, C);
+            if(iter == 0) {
+                if (verbose) Rprintf("Iteration %d: error is %f\n", iter + 1, C);
+            } 
             else {
                 total_time += (float) (end - start) / CLOCKS_PER_SEC;
-                Rprintf("Iteration %d: error is %f (50 iterations in %4.2f seconds)\n", iter, C, (float) (end - start) / CLOCKS_PER_SEC);
+                if (verbose) Rprintf("Iteration %d: error is %f (50 iterations in %4.2f seconds)\n", iter, C, (float) (end - start) / CLOCKS_PER_SEC);
             }
-			start = clock();
+            itercost[costi] = C;
+            itercost++;
+			  start = clock();
         }
     }
     end = clock(); total_time += (float) (end - start) / CLOCKS_PER_SEC;
     
+    if(exact) getCost(P, Y, N, no_dims, cost);
+    else      getCost(row_P, col_P, val_P, Y, N, no_dims, theta, cost);  // doing approximate computation here!
+    
+    
     // Clean up memory
     free(dY);
     free(uY);
@@ -182,7 +195,7 @@ void TSNE::run(double* X, int N, int D, double* Y, int no_dims, double perplexit
         free(col_P); col_P = NULL;
         free(val_P); val_P = NULL;
     }
-    Rprintf("Fitting performed in %4.2f seconds.\n", total_time);
+    if (verbose) Rprintf("Fitting performed in %4.2f seconds.\n", total_time);
 }
 
 
@@ -320,14 +333,94 @@ double TSNE::evaluateError(int* row_P, int* col_P, double* val_P, double* Y, int
     return C;
 }
 
+// Evaluate t-SNE cost function (exactly)
+void TSNE::getCost(double* P, double* Y, int N, int D, double* costs) {
+  
+  // Compute the squared Euclidean distance matrix
+  double* DD = (double*) malloc(N * N * sizeof(double));
+  double* Q = (double*) malloc(N * N * sizeof(double));
+  if(DD == NULL || Q == NULL) { Rcpp::stop("Memory allocation failed!\n"); }
+  computeSquaredEuclideanDistance(Y, N, D, DD);
+  
+  // Compute Q-matrix and normalization sum
+  double sum_Q = DBL_MIN;
+  for(int n = 0; n < N; n++) {
+    for(int m = 0; m < N; m++) {
+      if(n != m) {
+        Q[n * N + m] = 1 / (1 + DD[n * N + m]);
+        sum_Q += Q[n * N + m];
+      }
+      else Q[n * N + m] = DBL_MIN;
+    }
+  }
+  for(int i = 0; i < N * N; i++) Q[i] /= sum_Q;
+  
+  // Sum t-SNE error
+  for(int n = 0; n < N; n++) {
+    costs[n] = 0.0;
+    for(int m = 0; m < N; m++) {
+      costs[n] += P[n * N + m] * log((P[n * N + m] + 1e-9) / (Q[n * N + m] + 1e-9));
+    }
+  }
+  
+  // Clean up memory
+  free(DD);
+  free(Q);
+}
+
+// Evaluate t-SNE cost function (approximately)
+void TSNE::getCost(int* row_P, int* col_P, double* val_P, double* Y, int N, int D, double theta, double* costs)
+{
+  
+  // Get estimate of normalization term
+  SPTree* tree = new SPTree(D, Y, N);
+  double* buff = (double*) calloc(D, sizeof(double));
+  double sum_Q = .0;
+  for(int n = 0; n < N; n++) tree->computeNonEdgeForces(n, theta, buff, &sum_Q);
+  
+  // Loop over all edges to compute t-SNE error
+  int ind1, ind2;
+  double  Q;
+  for(int n = 0; n < N; n++) {
+    ind1 = n * D;
+    costs[n] = 0.0;
+    for(int i = row_P[n]; i < row_P[n + 1]; i++) {
+      Q = .0;
+      ind2 = col_P[i] * D;
+      for(int d = 0; d < D; d++) buff[d]  = Y[ind1 + d];
+      for(int d = 0; d < D; d++) buff[d] -= Y[ind2 + d];
+      for(int d = 0; d < D; d++) Q += buff[d] * buff[d];
+      Q = (1.0 / (1.0 + Q)) / sum_Q;
+      costs[n] += val_P[i] * log((val_P[i] + FLT_MIN) / (Q + FLT_MIN));
+    }
+  }
+  
+  // Clean up memory
+  free(buff);
+  delete tree;
+}
+
 
 // Compute input similarities with a fixed perplexity
-void TSNE::computeGaussianPerplexity(double* X, int N, int D, double* P, double perplexity) {
+void TSNE::computeGaussianPerplexity(double* X, int N, int D, double* P, double perplexity, bool distance_precomputed) {
 	
 	// Compute the squared Euclidean distance matrix
 	double* DD = (double*) malloc(N * N * sizeof(double));
     if(DD == NULL) { Rcpp::stop("Memory allocation failed!\n"); }
-	computeSquaredEuclideanDistance(X, N, D, DD);
+	
+	if (distance_precomputed) {
+	  DD = X;
+	} else {
+	  computeSquaredEuclideanDistance(X, N, D, DD);
+	}
+	
+  //Rprintf(" %4.4f \n", DD[0]);
+  //Rprintf(" %4.4f \n", DD[1]);
+  //Rprintf(" %4.4f \n", DD[2]);
+  // 	
+  //Rprintf(" %4.4f \n", X[0]);
+  //Rprintf(" %4.4f \n", X[1]);
+  //Rprintf(" %4.4f \n", X[2]);
 	
 	// Compute the Gaussian kernel row by row
 	for(int n = 0; n < N; n++) {
@@ -386,12 +479,13 @@ void TSNE::computeGaussianPerplexity(double* X, int N, int D, double* P, double
 	}
 	
 	// Clean up memory
-	free(DD); DD = NULL;
+	if (!distance_precomputed) { free(DD); }
+	DD = NULL;
 }
 
 
 // Compute input similarities with a fixed perplexity using ball trees (this function allocates memory another function should free)
-void TSNE::computeGaussianPerplexity(double* X, int N, int D, int** _row_P, int** _col_P, double** _val_P, double perplexity, int K) {
+void TSNE::computeGaussianPerplexity(double* X, int N, int D, int** _row_P, int** _col_P, double** _val_P, double perplexity, int K, bool verbose, bool distance_precomputed) {
     
     if(perplexity > K) Rprintf("Perplexity should be lower than K!\n");
     
@@ -409,18 +503,20 @@ void TSNE::computeGaussianPerplexity(double* X, int N, int D, int** _row_P, int*
     for(int n = 0; n < N; n++) row_P[n + 1] = row_P[n] + K;    
     
     // Build ball tree on data set
-    VpTree<DataPoint, euclidean_distance>* tree = new VpTree<DataPoint, euclidean_distance>();
-    vector<DataPoint> obj_X(N, DataPoint(D, -1, X));
-    for(int n = 0; n < N; n++) obj_X[n] = DataPoint(D, n, X + n * D);
-    tree->create(obj_X);
     
-    // Loop over all points to find nearest neighbors
-    Rprintf("Building tree...\n");
-    vector<DataPoint> indices;
-    vector<double> distances;
-    for(int n = 0; n < N; n++) {
+    if (distance_precomputed) {
+      VpTree<DataPoint, precomputed_distance>* tree = new VpTree<DataPoint, precomputed_distance>();
+      vector<DataPoint> obj_X(N, DataPoint(D, -1, X));
+      for(int n = 0; n < N; n++) obj_X[n] = DataPoint(D, n, X + n * D);
+      tree->create(obj_X);
+      
+      // Loop over all points to find nearest neighbors
+      if (verbose) Rprintf("Building tree...\n");
+      vector<DataPoint> indices;
+      vector<double> distances;
+      for(int n = 0; n < N; n++) {
         
-        if(n % 10000 == 0) Rprintf(" - point %d of %d\n", n, N);
+        if(n % 10000 == 0 && verbose) Rprintf(" - point %d of %d\n", n, N);
         
         // Find nearest neighbors
         indices.clear();
@@ -428,64 +524,143 @@ void TSNE::computeGaussianPerplexity(double* X, int N, int D, int** _row_P, int*
         tree->search(obj_X[n], K + 1, &indices, &distances);
         
         // Initialize some variables for binary search
-		bool found = false;
-		double beta = 1.0;
-		double min_beta = -DBL_MAX;
-		double max_beta =  DBL_MAX;
-		double tol = 1e-5;
-		
-		// Iterate until we found a good perplexity
-		int iter = 0; double sum_P;
-		while(!found && iter < 200) {
-			
-			// Compute Gaussian kernel row
-			for(int m = 0; m < K; m++) cur_P[m] = exp(-beta * distances[m + 1]);
-			
-			// Compute entropy of current row
-			sum_P = DBL_MIN;
-			for(int m = 0; m < K; m++) sum_P += cur_P[m];
-			double H = .0;
-			for(int m = 0; m < K; m++) H += beta * (distances[m + 1] * cur_P[m]);
-			H = (H / sum_P) + log(sum_P);
-			
-			// Evaluate whether the entropy is within the tolerance level
-			double Hdiff = H - log(perplexity);
-			if(Hdiff < tol && -Hdiff < tol) {
-				found = true;
-			}
-			else {
-				if(Hdiff > 0) {
-					min_beta = beta;
-					if(max_beta == DBL_MAX || max_beta == -DBL_MAX)
-						beta *= 2.0;
-					else
-						beta = (beta + max_beta) / 2.0;
-				}
-				else {
-					max_beta = beta;
-					if(min_beta == -DBL_MAX || min_beta == DBL_MAX)
-						beta /= 2.0;
-					else
-						beta = (beta + min_beta) / 2.0;
-				}
-			}
-			
-			// Update iteration counter
-			iter++;
-		}
-		
-		// Row-normalize current row of P and store in matrix
+        bool found = false;
+        double beta = 1.0;
+        double min_beta = -DBL_MAX;
+        double max_beta =  DBL_MAX;
+        double tol = 1e-5;
+        
+        // Iterate until we found a good perplexity
+        int iter = 0; double sum_P;
+        while(!found && iter < 200) {
+          
+          // Compute Gaussian kernel row
+          for(int m = 0; m < K; m++) cur_P[m] = exp(-beta * distances[m + 1]);
+          
+          // Compute entropy of current row
+          sum_P = DBL_MIN;
+          for(int m = 0; m < K; m++) sum_P += cur_P[m];
+          double H = .0;
+          for(int m = 0; m < K; m++) H += beta * (distances[m + 1] * cur_P[m]);
+          H = (H / sum_P) + log(sum_P);
+          
+          // Evaluate whether the entropy is within the tolerance level
+          double Hdiff = H - log(perplexity);
+          if(Hdiff < tol && -Hdiff < tol) {
+            found = true;
+          }
+          else {
+            if(Hdiff > 0) {
+              min_beta = beta;
+              if(max_beta == DBL_MAX || max_beta == -DBL_MAX)
+                beta *= 2.0;
+              else
+                beta = (beta + max_beta) / 2.0;
+            }
+            else {
+              max_beta = beta;
+              if(min_beta == -DBL_MAX || min_beta == DBL_MAX)
+                beta /= 2.0;
+              else
+                beta = (beta + min_beta) / 2.0;
+            }
+          }
+          
+          // Update iteration counter
+          iter++;
+        }
+        
+        // Row-normalize current row of P and store in matrix
         for(int m = 0; m < K; m++) cur_P[m] /= sum_P;
         for(int m = 0; m < K; m++) {
-            col_P[row_P[n] + m] = indices[m + 1].index();
-            val_P[row_P[n] + m] = cur_P[m];
+          col_P[row_P[n] + m] = indices[m + 1].index();
+          val_P[row_P[n] + m] = cur_P[m];
         }
+      }
+      
+      // Clean up memory
+      obj_X.clear();
+      free(cur_P);
+      delete tree;
+    } else {
+      VpTree<DataPoint, euclidean_distance>* tree =  new VpTree<DataPoint, euclidean_distance>();
+      vector<DataPoint> obj_X(N, DataPoint(D, -1, X));
+      for(int n = 0; n < N; n++) obj_X[n] = DataPoint(D, n, X + n * D);
+      tree->create(obj_X);
+      
+      // Loop over all points to find nearest neighbors
+      if (verbose) Rprintf("Building tree...\n");
+      vector<DataPoint> indices;
+      vector<double> distances;
+      for(int n = 0; n < N; n++) {
+        
+        if(n % 10000 == 0 && verbose) Rprintf(" - point %d of %d\n", n, N);
+        
+        // Find nearest neighbors
+        indices.clear();
+        distances.clear();
+        tree->search(obj_X[n], K + 1, &indices, &distances);
+        
+        // Initialize some variables for binary search
+        bool found = false;
+        double beta = 1.0;
+        double min_beta = -DBL_MAX;
+        double max_beta =  DBL_MAX;
+        double tol = 1e-5;
+        
+        // Iterate until we found a good perplexity
+        int iter = 0; double sum_P;
+        while(!found && iter < 200) {
+          
+          // Compute Gaussian kernel row
+          for(int m = 0; m < K; m++) cur_P[m] = exp(-beta * distances[m + 1]);
+          
+          // Compute entropy of current row
+          sum_P = DBL_MIN;
+          for(int m = 0; m < K; m++) sum_P += cur_P[m];
+          double H = .0;
+          for(int m = 0; m < K; m++) H += beta * (distances[m + 1] * cur_P[m]);
+          H = (H / sum_P) + log(sum_P);
+          
+          // Evaluate whether the entropy is within the tolerance level
+          double Hdiff = H - log(perplexity);
+          if(Hdiff < tol && -Hdiff < tol) {
+            found = true;
+          }
+          else {
+            if(Hdiff > 0) {
+              min_beta = beta;
+              if(max_beta == DBL_MAX || max_beta == -DBL_MAX)
+                beta *= 2.0;
+              else
+                beta = (beta + max_beta) / 2.0;
+            }
+            else {
+              max_beta = beta;
+              if(min_beta == -DBL_MAX || min_beta == DBL_MAX)
+                beta /= 2.0;
+              else
+                beta = (beta + min_beta) / 2.0;
+            }
+          }
+          
+          // Update iteration counter
+          iter++;
+        }
+        
+        // Row-normalize current row of P and store in matrix
+        for(int m = 0; m < K; m++) cur_P[m] /= sum_P;
+        for(int m = 0; m < K; m++) {
+          col_P[row_P[n] + m] = indices[m + 1].index();
+          val_P[row_P[n] + m] = cur_P[m];
+        }
+      }
+      
+      // Clean up memory
+      obj_X.clear();
+      free(cur_P);
+      delete tree;
     }
-    
-    // Clean up memory
-    obj_X.clear();
-    free(cur_P);
-    delete tree;
 }
 
 
